/* start of file CharPoly.cpp --------------------------------------------- */

/* <summary>
	SUBROUTINE CharPoly - Computes the Characteristic polynomial p of an
	n by n complex matrix a
   </summary>
*/

#include "stdafx.h"
#include "CharPoly.h"

using namespace std;

void CharPoly(complex<double> a[][maxA], int n, complex<double> p[])
/* p = Characteristic polynomial of an n by n complex matrix a, n > 1
*/
{
	int i, j, k, m;
	complex<double> s;
	complex<double> t0;
	complex<double> b[maxA][maxA];
	complex<double> c[maxA][maxA];
	complex<double> cZero = complex<double>(0.0, 0.0);
	complex<double> cOne = complex<double>(1.0, 0.0);
	if (n % 2 != 0)
		p[0] = -cOne;
	else
		p[0] = cOne;
	for (m = 0; m < n; m++)
	{
		if (m == 0)
		{
			for (i = 0; i < n; i++)
				for (j = 0; j < n; j++)
					c[i][j] = a[i][j];
		}
		else
		{
			for (i = 0; i < n; i++)
			{
				for (j = 0; j < n; j++)
				{
					s = cZero;
					for (k = 0; k < n; k++)
						s += b[i][k] * a[k][j];
					c[i][j] = s;
				}
			}
		}
		t0 = 0;
		for (i = 0; i < n; i++)		/* t0 = Trace(c) */
			t0 += c[i][i];
		t0 /= (double)(m + 1);
		p[m + 1] = -t0 * p[0];
		if (m < n - 1)
			for (i = 0; i < n; i++)
				for (j = 0; j < n; j++)
					if (j == i)
						b[i][j] = c[i][j] - t0;
					else
						b[i][j] = c[i][j];
	}
	if (n % 2 != 0)
	{
		for (i = 0; i <= n; i++)	/* p = -p */
			p[i] = -p[i];
	}
} /* CharPoly */

/* end of file CharPoly.cpp ----------------------------------------------- */

